package lightcontainer.domains.client;

import lightcontainer.interfaces.StoreProcessorFFE;

import java.io.*;
import java.net.Socket;
import java.nio.charset.StandardCharsets;
import java.util.Objects;

/**
 * StoreProcessor
 *
 * <!> MULTICAST CLIENT <!>
 * Class communicating with the storebackend and
 * processing the tasks in the FileFrontEnd
 *
 * @version     1.1
 * @since       1.0
 *
 * @see         Runnable
 * @see         AutoCloseable
 * @author      Jérémi NIHART <j.nihart@student.helmo.be>
 */
public class StoreProcessor implements Runnable, AutoCloseable {
    // Variables
    private final StoreProcessorFFE fileFrontEnd;
    private final Socket store;
    private final String domain;
    private boolean client_run;

    private BufferedReader reader;
    private PrintWriter writer;

    // Constructor
    public StoreProcessor(Socket socket, String domain, StoreProcessorFFE ffe) {
        this.domain = domain;
        this.fileFrontEnd = ffe;
        this.store = socket;
        this.client_run = false;
        initStore();
    }

    /**
     * Initialise the Store's Reader and Writer.
     *
     * @see BufferedReader
     * @see PrintWriter
     * @since 1.0
     */
    private void initStore() {
        try {
            this.reader = new BufferedReader(new InputStreamReader(
                    this.store.getInputStream(),
                    StandardCharsets.UTF_8
            ));
            this.writer = new PrintWriter(new OutputStreamWriter(
                    this.store.getOutputStream(),
                    StandardCharsets.UTF_8
            ), true);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * Thread Function
     * Start the dialogue with the storebackend.
     *
     * @since 1.0
     */
    @Override
    public void run() {
        this.client_run = true;
        while (this.client_run) {
            try {
                String command = this.reader.readLine();
                // TODO gestion de la réception de commandes, fichier, ...
                if (command != null) System.out.println("StoreBackEnd: " + command);
            } catch (IOException ignore) { }
        }
    }

    /**
     * AutoClosable Function
     * Close the Storage thread and resources.
     *
     * @since 1.0
     */
    @Override
    public void close() {
        if (this.client_run) {
            try {
                this.client_run = false;
                this.store.close();
            } catch (IOException ignored) { }
        }
    }

    /**
     * Compare two StoreProcessor object to check if equals.
     */
    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        StoreProcessor that = (StoreProcessor) o;
        return Objects.equals(domain, that.domain);
    }

    /**
     * Get hash code
     */
    @Override
    public int hashCode() {
        return Objects.hash(domain);
    }
}
