package lightcontainer.domains.server;

import lightcontainer.domains.client.ClientHandler;
import lightcontainer.interfaces.ProtocolRepository;
import lightcontainer.interfaces.UnicastCHR;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;

public class UnicastServerListener {
    // Variables
    private ServerSocket server;
    private final UnicastCHR repository;
    private ProtocolRepository protocolRep;
    private final int server_port;
    private boolean server_run;

    // Constructor
    public UnicastServerListener(UnicastCHR repository, ProtocolRepository protocolRep, int port) {
        this.repository = repository;
        this.protocolRep = protocolRep;
        this.server_port = port;
        this.server_run = false;
        repository.setServerListener(this);
    }

    /**
     * Initializes the server and starts it on the previously selected port.
     *
     * @since       1.0
     *
     * @see         Thread#start()
     * @see         ClientHandler
     */
    public void start() {
        try {
            // Allow looping in the loop and create a socket server
            this.server_run = true;
            this.server = new ServerSocket(this.server_port);
            while (this.server_run) {
                // Accepting connection requests (blocking)
                Socket client = this.server.accept();
                // Create a new Handler client by passing these dependencies to it
                ClientHandler clientHandler = new ClientHandler(client, null, protocolRep); // TODO passer FileFrontEnd ou faire ca dans le repository ?!
                // Add the client handler to its repository (clienthandlerrepository)
                // this.repository.add(clientHandler); TODO REPOSITORY
                // Start the thread
                (new Thread(clientHandler)).start();
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * Stops the server and terminates the new connection.
     *
     * @since       1.0
     */
    public void stop() {
        if (this.server_run) {
            try {
                this.server_run = false;
                this.server.close();
            } catch (IOException ignored) { }
        }
    }
}
